from typing import Tuple, List
from thai_sentiment import __file__ as thai_sentiment_file
from pathlib import Path
import os
from .architecture import NbSvmClassifier
from sklearn.feature_extraction.text import TfidfVectorizer
from pythainlp.ulmfit import process_thai
import pickle
import urllib.request


class CustomUnpickler(pickle.Unpickler):
    def find_class(self, module, name):
        if name == 'NbSvmClassifier':
            from .architecture import NbSvmClassifier
            return NbSvmClassifier
        return super().find_class(module, name)


thai_sentiment_path = Path(os.path.dirname(thai_sentiment_file))
labels = ["pos", "neu", "neg"]

# MODEL_URL = "https://github.com/cstorm125/thai_sentiment/raw/main/thai_sentiment/pretrained/nbsvm_c3_l2_dual.pkl"
# ENC_URL = "https://github.com/cstorm125/thai_sentiment/raw/main/thai_sentiment/pretrained/tfidf_wisesight_sentiment.pkl"
# urllib.request.urlretrieve(MODEL_URL, thai_sentiment_path / 'pretrained/nbsvm_c3_l2_dual.pkl')
# urllib.request.urlretrieve(ENC_URL, thai_sentiment_path / 'pretrained/tfidf_wisesight_sentiment.pkl')

with open(thai_sentiment_path / 'pretrained/tfidf_wisesight_sentiment.pkl', 'rb') as f:
    tfidf = pickle.load(f)
models = CustomUnpickler(open(thai_sentiment_path / 'pretrained/nbsvm_c3_l2_dual.pkl', 'rb')).load()


def get_sentiment(text: str)->Tuple[str, List[float]]:
    probs = {'pos': 0., 'neu': 0., 'neg': 0.}
    for i in range(len(labels)):
        probs[labels[i]] = models[i].predict_proba(tfidf.transform([text]))[:, 1][0]
    return max(probs, key=probs.get), probs



from setuptools import setup

setup(
    name='thai_sentiment',
    packages=['thai_sentiment'],
    version='v0.1.3',  # Ideally should be same as your GitHub release tag varsion
    description='The naive sentiment classification function based on NBSVM trained on wisesight_sentiment',
    author='cstorm125',
    author_email='cebril@gmail.com',
    url='https://github.com/cstorm125/thai_sentiment',
    download_url='https://github.com/cstorm125/thai_sentiment/archive/refs/tags/v0.1.3.tar.gz',
    keywords=['sentiment analysis', 'thai', 'nlp'],
    classifiers=[],
    install_requires=[
        'pythainlp',
        'sklearn',
        'scipy',
    ],
    include_package_data=True,
)



from sklearn.base import BaseEstimator, ClassifierMixin
from sklearn.utils.validation import check_X_y, check_is_fitted
from sklearn.linear_model import LogisticRegression
from scipy import sparse

class NbSvmClassifier(BaseEstimator, ClassifierMixin):
    def __init__(self, penalty='l2', C=1.0, dual=False, seed=1412):
        self.penalty = penalty
        self.C = C
        self.dual = dual
        self.seed = seed
        
    def predict(self, x):
        # Verify that model has been fit
        check_is_fitted(self, ['_r', '_clf'])
        return self._clf.predict(x.multiply(self._r))

    def predict_proba(self, x):
        # Verify that model has been fit
        check_is_fitted(self, ['_r', '_clf'])
        return self._clf.predict_proba(x.multiply(self._r))

    def fit(self, x, y):
        # Check that X and y have correct shape
        y = y.toarray().ravel() if type(y)!=np.ndarray else y.ravel()
        x, y = check_X_y(x, y, accept_sparse=True)

        def pr(x, y_i, y):
            p = x[y==y_i].sum(0)
            return (p+1) / ((y==y_i).sum()+1)

        self._r = sparse.csr_matrix(np.log(pr(x,1,y) / pr(x,0,y)))
        x_nb = x.multiply(self._r)
        self._clf = LogisticRegression(penalty = self.penalty, 
                                       C=self.C, 
                                       dual=self.dual,
                                       solver='liblinear',
                                       random_state=self.seed,).fit(x_nb, y)
        return self